@@ -13,7 +13,8 @@ use serde::{Deserialize, Serialize};
 use std::ops::RangeBounds;
 
 use backend::{
-    MapxRawVersionedIter, RawKey, RawValue, ValueMut, VerSig, INITIAL_BRANCH_NAME, NULL,
+    BytesKey, BytesValue, MapxRawVersionedIter, ValueMut, VerSig, INITIAL_BRANCH_NAME,
+    NULL,
 };
 
 /// Advanced `MapxRaw`, with versioned feature.
@@ -39,24 +40,28 @@ impl MapxRawVersioned {
 
     /// Insert a KV to the head version of the default branch.
     #[inline(always)]
-    pub fn insert(&mut self, key: RawKey, value: RawValue) -> Result<Option<RawValue>> {
+    pub fn insert_ref(
+        &mut self,
+        key: &[u8],
+        value: &[u8],
+    ) -> Result<Option<BytesValue>> {
         self.inner.insert(key, value).c(d!())
     }
 
     /// Insert a KV to the head version of a specified branch.
     #[inline(always)]
-    pub fn insert_by_branch(
+    pub fn insert_ref_by_branch(
         &mut self,
-        key: RawKey,
-        value: RawValue,
+        key: &[u8],
+        value: &[u8],
         branch_id: BranchID,
-    ) -> Result<Option<RawValue>> {
+    ) -> Result<Option<BytesValue>> {
         self.inner.insert_by_branch(key, value, branch_id).c(d!())
     }
 
     /// Remove a KV from the head version of the default branch.
     #[inline(always)]
-    pub fn remove(&mut self, key: &[u8]) -> Result<Option<RawValue>> {
+    pub fn remove(&mut self, key: &[u8]) -> Result<Option<BytesValue>> {
         self.inner.remove(key).c(d!())
     }
 
@@ -66,20 +71,23 @@ impl MapxRawVersioned {
         &mut self,
         key: &[u8],
         branch_id: BranchID,
-    ) -> Result<Option<RawValue>> {
+    ) -> Result<Option<BytesValue>> {
         self.inner.remove_by_branch(key, branch_id).c(d!())
     }
 
     /// Get the value of a key from the default branch.
     #[inline(always)]
-    pub fn get(&self, key: &[u8]) -> Option<RawValue> {
+    pub fn get(&self, key: &[u8]) -> Option<BytesValue> {
         self.inner.get(key)
     }
 
     /// Get the value of a key from the head of a specified branch.
     #[inline(always)]
-    pub fn get_by_branch(&self, key: &[u8], branch_name: &[u8]) -> Option<RawValue> {
-        let branch_id = self.inner.branch_name_to_branch_id._get(branch_name)?;
+    pub fn get_by_branch(&self, key: &[u8], branch_name: &[u8]) -> Option<BytesValue> {
+        let branch_id = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)?;
         self.inner.get_by_branch(key, branch_id)
     }
 
@@ -90,9 +98,15 @@ impl MapxRawVersioned {
         key: &[u8],
         branch_name: &[u8],
         version_name: &[u8],
-    ) -> Option<RawValue> {
-        let branch_id = self.inner.branch_name_to_branch_id._get(branch_name)?;
-        let version_id = self.inner.version_name_to_version_id._get(version_name)?;
+    ) -> Option<BytesValue> {
+        let branch_id = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)?;
+        let version_id = self
+            .inner
+            .version_name_to_version_id
+            .get_ref_bytes_k(version_name)?;
         self.inner.get_by_branch_version(key, branch_id, version_id)
     }
 
@@ -109,7 +123,10 @@ impl MapxRawVersioned {
         key: &[u8],
         branch_name: &[u8],
     ) -> Option<ValueMut<'_>> {
-        let branch_id = self.inner.branch_name_to_branch_id._get(branch_name)?;
+        let branch_id = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)?;
         self.inner.get_mut_by_branch(key, branch_id)
     }
 
@@ -117,7 +134,7 @@ impl MapxRawVersioned {
     /// if the target key does not exist, will try to
     /// search a closest value bigger than the target key.
     #[inline(always)]
-    pub fn get_ge(&self, key: &[u8]) -> Option<(RawKey, RawValue)> {
+    pub fn get_ge(&self, key: &[u8]) -> Option<(BytesKey, BytesValue)> {
         self.inner.get_ge(key)
     }
 
@@ -129,8 +146,11 @@ impl MapxRawVersioned {
         &self,
         key: &[u8],
         branch_name: &[u8],
-    ) -> Option<(RawKey, RawValue)> {
-        let branch_id = self.inner.branch_name_to_branch_id._get(branch_name)?;
+    ) -> Option<(BytesKey, BytesValue)> {
+        let branch_id = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)?;
         self.inner.get_ge_by_branch(key, branch_id)
     }
 
@@ -143,9 +163,15 @@ impl MapxRawVersioned {
         key: &[u8],
         branch_name: &[u8],
         version_name: &[u8],
-    ) -> Option<(RawKey, RawValue)> {
-        let branch_id = self.inner.branch_name_to_branch_id._get(branch_name)?;
-        let version_id = self.inner.version_name_to_version_id._get(version_name)?;
+    ) -> Option<(BytesKey, BytesValue)> {
+        let branch_id = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)?;
+        let version_id = self
+            .inner
+            .version_name_to_version_id
+            .get_ref_bytes_k(version_name)?;
         self.inner
             .get_ge_by_branch_version(key, branch_id, version_id)
     }
@@ -154,7 +180,7 @@ impl MapxRawVersioned {
     /// if the target key does not exist, will try to
     /// search a closest value less than the target key.
     #[inline(always)]
-    pub fn get_le(&self, key: &[u8]) -> Option<(RawKey, RawValue)> {
+    pub fn get_le(&self, key: &[u8]) -> Option<(BytesKey, BytesValue)> {
         self.inner.get_le(key)
     }
 
@@ -166,8 +192,11 @@ impl MapxRawVersioned {
         &self,
         key: &[u8],
         branch_name: &[u8],
-    ) -> Option<(RawKey, RawValue)> {
-        let branch_id = self.inner.branch_name_to_branch_id._get(branch_name)?;
+    ) -> Option<(BytesKey, BytesValue)> {
+        let branch_id = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)?;
         self.inner.get_le_by_branch(key, branch_id)
     }
 
@@ -180,9 +209,15 @@ impl MapxRawVersioned {
         key: &[u8],
         branch_name: &[u8],
         version_name: &[u8],
-    ) -> Option<(RawKey, RawValue)> {
-        let branch_id = self.inner.branch_name_to_branch_id._get(branch_name)?;
-        let version_id = self.inner.version_name_to_version_id._get(version_name)?;
+    ) -> Option<(BytesKey, BytesValue)> {
+        let branch_id = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)?;
+        let version_id = self
+            .inner
+            .version_name_to_version_id
+            .get_ref_bytes_k(version_name)?;
         self.inner
             .get_le_by_branch_version(key, branch_id, version_id)
     }
@@ -199,7 +234,7 @@ impl MapxRawVersioned {
         let branch_id = self
             .inner
             .branch_name_to_branch_id
-            ._get(branch_name)
+            .get_ref_bytes_k(branch_name)
             .unwrap_or(NULL);
         self.inner.iter_by_branch(branch_id)
     }
@@ -214,13 +249,13 @@ impl MapxRawVersioned {
         let branch_id = self
             .inner
             .branch_name_to_branch_id
-            ._get(branch_name)
+            .get_ref_bytes_k(branch_name)
             .unwrap_or(NULL);
 
         let version_id = self
             .inner
             .version_name_to_version_id
-            ._get(version_name)
+            .get_ref_bytes_k(version_name)
             .unwrap_or(NULL);
 
         self.inner.iter_by_branch_version(branch_id, version_id)
@@ -228,21 +263,24 @@ impl MapxRawVersioned {
 
     /// Create a range iterator over the default branch.
     #[inline(always)]
-    pub fn range<R: RangeBounds<RawKey>>(&self, bounds: R) -> MapxRawVersionedIter {
+    pub fn range<'a, R: RangeBounds<&'a [u8]>>(
+        &'a self,
+        bounds: R,
+    ) -> MapxRawVersionedIter<'a> {
         self.inner.range(bounds)
     }
 
     /// Create a range iterator over a specified branch.
     #[inline(always)]
-    pub fn range_by_branch<R: RangeBounds<RawKey>>(
-        &self,
+    pub fn range_by_branch<'a, R: RangeBounds<&'a [u8]>>(
+        &'a self,
         branch_name: &[u8],
         bounds: R,
-    ) -> MapxRawVersionedIter {
+    ) -> MapxRawVersionedIter<'a> {
         let branch_id = self
             .inner
             .branch_name_to_branch_id
-            ._get(branch_name)
+            .get_ref_bytes_k(branch_name)
             .unwrap_or(NULL);
 
         self.inner.range_by_branch(branch_id, bounds)
@@ -250,22 +288,22 @@ impl MapxRawVersioned {
 
     /// Create a range iterator over a specified version of a specified branch.
     #[inline(always)]
-    pub fn range_by_branch_version<R: RangeBounds<RawKey>>(
-        &self,
+    pub fn range_by_branch_version<'a, R: RangeBounds<&'a [u8]>>(
+        &'a self,
         branch_name: &[u8],
         version_name: &[u8],
         bounds: R,
-    ) -> MapxRawVersionedIter {
+    ) -> MapxRawVersionedIter<'a> {
         let branch_id = self
             .inner
             .branch_name_to_branch_id
-            ._get(branch_name)
+            .get_ref_bytes_k(branch_name)
             .unwrap_or(NULL);
 
         let version_id = self
             .inner
             .version_name_to_version_id
-            ._get(version_name)
+            .get_ref_bytes_k(version_name)
             .unwrap_or(NULL);
 
         self.inner
@@ -305,12 +343,15 @@ impl MapxRawVersioned {
     /// Get the total number of items of the head of a specified branch.
     #[inline(always)]
     pub fn len_by_branch(&self, branch_name: &[u8]) -> usize {
-        let branch_id =
-            if let Some(id) = self.inner.branch_name_to_branch_id._get(branch_name) {
-                id
-            } else {
-                return 0;
-            };
+        let branch_id = if let Some(id) = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)
+        {
+            id
+        } else {
+            return 0;
+        };
         self.inner.len_by_branch(branch_id)
     }
 
@@ -321,18 +362,24 @@ impl MapxRawVersioned {
         branch_name: &[u8],
         version_name: &[u8],
     ) -> usize {
-        let branch_id =
-            if let Some(id) = self.inner.branch_name_to_branch_id._get(branch_name) {
-                id
-            } else {
-                return 0;
-            };
-        let version_id =
-            if let Some(id) = self.inner.version_name_to_version_id._get(version_name) {
-                id
-            } else {
-                return 0;
-            };
+        let branch_id = if let Some(id) = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)
+        {
+            id
+        } else {
+            return 0;
+        };
+        let version_id = if let Some(id) = self
+            .inner
+            .version_name_to_version_id
+            .get_ref_bytes_k(version_name)
+        {
+            id
+        } else {
+            return 0;
+        };
         self.inner.len_by_branch_version(branch_id, version_id)
     }
 
@@ -381,7 +428,7 @@ impl MapxRawVersioned {
         let branch_id = self
             .inner
             .branch_name_to_branch_id
-            ._get(branch_name)
+            .get_ref_bytes_k(branch_name)
             .c(d!("branch not found"))?;
 
         self.inner
@@ -392,12 +439,15 @@ impl MapxRawVersioned {
     /// Check if a verison exists on default branch.
     #[inline(always)]
     pub fn version_exists(&self, version_name: &[u8]) -> bool {
-        let version_id =
-            if let Some(id) = self.inner.version_name_to_version_id._get(version_name) {
-                id
-            } else {
-                return false;
-            };
+        let version_id = if let Some(id) = self
+            .inner
+            .version_name_to_version_id
+            .get_ref_bytes_k(version_name)
+        {
+            id
+        } else {
+            return false;
+        };
 
         self.inner.version_exists(version_id)
     }
@@ -409,19 +459,25 @@ impl MapxRawVersioned {
         version_name: &[u8],
         branch_name: &[u8],
     ) -> bool {
-        let version_id =
-            if let Some(id) = self.inner.version_name_to_version_id._get(version_name) {
-                id
-            } else {
-                return false;
-            };
+        let version_id = if let Some(id) = self
+            .inner
+            .version_name_to_version_id
+            .get_ref_bytes_k(version_name)
+        {
+            id
+        } else {
+            return false;
+        };
 
-        let branch_id =
-            if let Some(id) = self.inner.branch_name_to_branch_id._get(branch_name) {
-                id
-            } else {
-                return false;
-            };
+        let branch_id = if let Some(id) = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)
+        {
+            id
+        } else {
+            return false;
+        };
 
         self.inner.version_exists_on_branch(version_id, branch_id)
     }
@@ -439,19 +495,25 @@ impl MapxRawVersioned {
         version_name: &[u8],
         branch_name: &[u8],
     ) -> bool {
-        let version_id =
-            if let Some(id) = self.inner.version_name_to_version_id._get(version_name) {
-                id
-            } else {
-                return false;
-            };
+        let version_id = if let Some(id) = self
+            .inner
+            .version_name_to_version_id
+            .get_ref_bytes_k(version_name)
+        {
+            id
+        } else {
+            return false;
+        };
 
-        let branch_id =
-            if let Some(id) = self.inner.branch_name_to_branch_id._get(branch_name) {
-                id
-            } else {
-                return false;
-            };
+        let branch_id = if let Some(id) = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)
+        {
+            id
+        } else {
+            return false;
+        };
 
         self.inner.version_created_on_branch(version_id, branch_id)
     }
@@ -480,7 +542,7 @@ impl MapxRawVersioned {
         let branch_id = self
             .inner
             .branch_name_to_branch_id
-            ._get(branch_name)
+            .get_ref_bytes_k(branch_name)
             .c(d!("branch not found"))?;
 
         self.inner.version_pop_by_branch(branch_id).c(d!())
@@ -502,7 +564,7 @@ impl MapxRawVersioned {
         let base_branch_id = self
             .inner
             .branch_name_to_branch_id
-            ._get(base_branch_name)
+            .get_ref_bytes_k(base_branch_name)
             .c(d!("base branch not found"))?;
 
         self.inner
@@ -513,7 +575,11 @@ impl MapxRawVersioned {
     /// Check if a branch exists or not.
     #[inline(always)]
     pub fn branch_exists(&self, branch_name: &[u8]) -> bool {
-        if let Some(branch_id) = self.inner.branch_name_to_branch_id._get(branch_name) {
+        if let Some(branch_id) = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)
+        {
             self.inner.branch_exists(branch_id)
         } else {
             false
@@ -529,7 +595,11 @@ impl MapxRawVersioned {
     /// and should not do any tracing.
     #[inline(always)]
     pub fn branch_remove(&mut self, branch_name: &[u8]) -> Result<()> {
-        if let Some(branch_id) = self.inner.branch_name_to_branch_id._get(branch_name) {
+        if let Some(branch_id) = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)
+        {
             self.inner.branch_remove(branch_id).c(d!())
         } else {
             Err(eg!("branch not found"))
@@ -548,7 +618,7 @@ impl MapxRawVersioned {
         let branch_id = self
             .inner
             .branch_name_to_branch_id
-            ._get(branch_name)
+            .get_ref_bytes_k(branch_name)
             .c(d!("branch not found"))?;
         self.inner.branch_truncate(branch_id).c(d!())
     }
@@ -569,12 +639,12 @@ impl MapxRawVersioned {
         let branch_id = self
             .inner
             .branch_name_to_branch_id
-            ._get(branch_name)
+            .get_ref_bytes_k(branch_name)
             .c(d!("branch not found"))?;
         let last_version_id = self
             .inner
             .version_name_to_version_id
-            ._get(last_version_name)
+            .get_ref_bytes_k(last_version_name)
             .c(d!("version not found"))?;
         self.inner
             .branch_truncate_to(branch_id, last_version_id)
@@ -593,7 +663,7 @@ impl MapxRawVersioned {
         let branch_id = self
             .inner
             .branch_name_to_branch_id
-            ._get(branch_name)
+            .get_ref_bytes_k(branch_name)
             .c(d!("branch not found"))?;
         self.inner.branch_pop_version(branch_id).c(d!())
     }
@@ -604,7 +674,7 @@ impl MapxRawVersioned {
         let branch_id = self
             .inner
             .branch_name_to_branch_id
-            ._get(branch_name)
+            .get_ref_bytes_k(branch_name)
             .c(d!("branch not found"))?;
         self.inner.branch_merge_to_parent(branch_id).c(d!())
     }
@@ -612,7 +682,11 @@ impl MapxRawVersioned {
     /// Check if a branch has children branches.
     #[inline(always)]
     pub fn branch_has_children(&self, branch_name: &[u8]) -> bool {
-        if let Some(id) = self.inner.branch_name_to_branch_id._get(branch_name) {
+        if let Some(id) = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)
+        {
             self.inner.branch_has_children(id)
         } else {
             false
@@ -628,7 +702,10 @@ impl MapxRawVersioned {
     /// Get the signature of the head of a specified branch.
     #[inline(always)]
     pub fn sig_get_by_branch(&self, branch_name: &[u8]) -> Option<VerSig> {
-        let branch_id = self.inner.branch_name_to_branch_id._get(branch_name)?;
+        let branch_id = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)?;
         self.inner.sig_get_by_branch(branch_id)
     }
 
@@ -639,8 +716,14 @@ impl MapxRawVersioned {
         branch_name: &[u8],
         version_name: &[u8],
     ) -> Option<VerSig> {
-        let branch_id = self.inner.branch_name_to_branch_id._get(branch_name)?;
-        let version_id = self.inner.version_name_to_version_id._get(version_name)?;
+        let branch_id = self
+            .inner
+            .branch_name_to_branch_id
+            .get_ref_bytes_k(branch_name)?;
+        let version_id = self
+            .inner
+            .version_name_to_version_id
+            .get_ref_bytes_k(version_name)?;
         self.inner
             .sig_get_by_branch_version(branch_id, Some(version_id))
     }
@@ -661,7 +744,7 @@ impl MapxRawVersioned {
         let branch_id = self
             .inner
             .branch_name_to_branch_id
-            ._get(branch_name)
+            .get_ref_bytes_k(branch_name)
             .c(d!())?;
         self.inner
             .prune_by_branch(branch_id, reserved_ver_num)
